#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>

#include "parse.h"
#include "hashtable.h"
#include "separate.h"

/* Usage:
   cat text.txt | ./separate
   echo "thisissometext" | ./separate
*/

/* ------------------------------------------------------------------------- *
 * Read stdin until EOF and returns a buffer with its contents.
 *
 * The buffer must later be freed.
 *
 * PARAMETERS
 * none
 *
 * RETURN
 * char *       A new pointer to a character string.
 * NULL         If an error has occurred.
 * ------------------------------------------------------------------------- */
static char *
readstdin(void)
{
  int c;
  char *buffer;
  size_t n = 0, allocated = 1024;
  if (!(buffer = calloc(allocated, sizeof(char *))))
    {
      fprintf(stderr, "malloc error\n");
      return NULL;
    }
  while ((c = getc(stdin)) != EOF)
    {
      if (n >= allocated)
	{
	  allocated *= 2;
	  if (!(buffer = realloc(buffer, allocated * sizeof(char *))))
	    {
	      fprintf(stderr, "realloc error\n");
	      return NULL;
	    }
	}
      buffer[n++] = c;
    }
  // Ensure everything is initialized
  for(size_t i=n; i < allocated; i++)
    buffer[i] = '\0';
  return buffer;
}

int
main(void)
{
  // Load words.csv and compute total number of elements
  unsigned long long sum = 0;
  strtable *table = parse("words.csv");
  if (!table)
    {
      printf("Error parsing 'words.csv'\n");
      exit(-1);
    }
  for (size_t i = 0; i < table->n; i++)
      sum += table->nboccur[i];

  // Create hash table to store the *log* frequencies of the words
  double log_sum = log(sum);
  hashtable *log_frequencies = createhashtable(table->n);
  if (!log_frequencies)
    {
      printf("Error creating the hashtable\n");
      exit(-1);
    }
  for (size_t i = 0; i < table->n; i++)
      insertelement(log_frequencies, table->string[i],
                    log(table->nboccur[i]) - log_sum);



  char *output, *input = readstdin();
  if (!input)
    {
      fprintf(stderr, "Error reading input\n");
      return -1;
    }

  output = separate(input, log_frequencies);
  if (!output)
    {
      fprintf(stderr, "Error processing input\n");
      return -1;
    }
  printf("%s\n", output);

  free(output);
  free(input);
  freehashtable(log_frequencies);
  freestrtable(table);

  return 0;
}

